// TYPE: C
// PRJ: JCL
// PGM: Jiro_Array
// VER: 2.00
// USES: GCC
// CREATE: 2002/09/02( Shiratori )
// UPDATE: 2007/04/02( tamura )
/////////////////////////////////////////////////////
// ＯＳ依存関数をなくし、ＡＮＳＩ準拠関数で作成。
// バージョン１に比べ、メモリ確保速度なども向上している。

// MFCプロジェクトを使用する場合は、下記のコメントを外す。
//#include "stdafx.h"

#include "Jiro_Array.h"
#include <stdlib.h>
#include <string.h>

//*************************************************************************
// 内部関数のプロトタイプ宣言
//*************************************************************************
static bool IsValidIndex( JIRO_ARY_HDL inAryHdl, long inIndex );
static void* GetDataPosByIndex( void* inRecPtr, long inRecSize, long inIndex );

//*************************************************************************
// 関数
//*************************************************************************
// -------------------------------------------------
// 関数 : Jiro_Ary_Init
// 機能 : 動的配列を初期化
// -------------------------------------------------
JIRO_ARY_HDL Jiro_Ary_Init( long inRecSize, long initNumOfRecs, long inExpandNumOfRecs )
{
	JIRO_ARY*	aryPtr;
	long		initRecNum;
	long		expandRecNum;

	// 初期レコード数
	initRecNum = kDefaultInitRecNum;
	if( 0 < initNumOfRecs )
	{
		initRecNum = initNumOfRecs;	// 引数の値を使う
	}

	// 増分レコード数を設定
	expandRecNum = kDefaultExpandRecNum;
	if( 0 < inExpandNumOfRecs )
	{
		expandRecNum = inExpandNumOfRecs;	// 引数の値を使う
	}

	// 配列管理テーブルの確保
	aryPtr = ( JIRO_ARY* )malloc( sizeof( JIRO_ARY ) );
	if( aryPtr == NULL )
	{
		return NULL;	// 確保できなかった
	}

	// 初期化
	aryPtr->recPtr = NULL;
	aryPtr->recNum = 0;
	aryPtr->dataNum = 0;
	aryPtr->recSize = inRecSize;
	aryPtr->initRecNum = initRecNum;
	aryPtr->expandRecNum = expandRecNum;

	return aryPtr;
}

//--------------------------------------------------
// 関数 : Jiro_Ary_Free
// 機能 : 動的配列を解放
//--------------------------------------------------
void Jiro_Ary_Free( JIRO_ARY_HDL inAryHdl )
{
	// ＮＵＬＬチェック
	if( inAryHdl == NULL )
	{
		return;	// 配列管理テーブル未確保
	}

	// データ領域を確保していたら解放
	if( inAryHdl->recPtr != NULL )
	{
		free( inAryHdl->recPtr );
	}

	// 配列管理テーブル解放
	memset( inAryHdl, 0, sizeof( JIRO_ARY ) );  // 念のためクリア
	free( inAryHdl );
}

// -------------------------------------------------
// 関数 : Jiro_Ary_GetSize
// 機能 : レコード（データ）数を取得
// -------------------------------------------------
long Jiro_Ary_GetSize( JIRO_ARY_HDL inAryHdl )
{
	return inAryHdl->dataNum;
}

// -------------------------------------------------
// 関数 : Jiro_Ary_GetData
// 機能 : レコード（データ）を取得
// -------------------------------------------------
void Jiro_Ary_GetData( JIRO_ARY_HDL inAryHdl, long inIndex, void* outRecPtr )
{
	void* dataPos;

	// 取り出し先を0クリア
	memset( outRecPtr, 0, inAryHdl->recSize );

	if( inAryHdl->recPtr == NULL )
	{ // データがまだない
		return;
	}
	if( IsValidIndex( inAryHdl, inIndex ) == false )
	{ // インデックスが範囲外
		return;
	}

	// データ位置
	dataPos = GetDataPosByIndex( inAryHdl->recPtr, inAryHdl->recSize, inIndex );

	// データ取得
	memcpy( outRecPtr, dataPos, inAryHdl->recSize );
}

// -------------------------------------------------
// 関数 : Jiro_Ary_Insert
// 機能 : レコード（データ）を挿入
// 戻り値(0: 成功、-1: メモリ不足エラー)
// -------------------------------------------------
int Jiro_Ary_Insert( JIRO_ARY_HDL inAryHdl, long inIndex, void* inInsertRecPtr )
{
	void*	newBufPtr;
	long	newBufSize;
	long	newRecNum;
	long	dataIndex;
	void*	newPos;
	void*	oldPos;
	long	shiftDataSize;
	void*	newDataPos;

	// レコードバッファをまだ確保していない？
	if( inAryHdl->recPtr == NULL )
	{
		// 新しいレコード数
		newRecNum = inAryHdl->initRecNum;

		// 新しいバッファサイズ
		newBufSize = newRecNum * inAryHdl->recSize;

		// 領域確保
		newBufPtr = malloc( newBufSize );
		if( newBufPtr == NULL )
		{ // 確保失敗
			inAryHdl->recPtr  = NULL;
			inAryHdl->recNum  = 0;
			inAryHdl->dataNum = 0;
			return -1;  // エラー
		}

		// 配列管理テーブルを更新
		inAryHdl->recPtr  = newBufPtr;
		inAryHdl->recNum  = newRecNum;
		inAryHdl->dataNum = 0;
	}

	// 空きがない？
	if( inAryHdl->recNum <= inAryHdl->dataNum )
	{
		// 新しいレコード数
		newRecNum = inAryHdl->recNum + inAryHdl->expandRecNum;

		// 新しいバッファサイズ
		newBufSize = newRecNum * inAryHdl->recSize;

		// バッファを拡張
		newBufPtr = realloc( inAryHdl->recPtr, newBufSize );
		if( newBufPtr == NULL )
		{ // 拡張できなかった
			return -1;  // エラー
		}

		// 配列管理テーブルを更新
		inAryHdl->recPtr = newBufPtr;
		inAryHdl->recNum = newRecNum;
	}

	// レコードの挿入位置を補正
	dataIndex = inIndex;
	if( dataIndex < 0 )
	{
		dataIndex = 0;
	}
	if( inAryHdl->dataNum < dataIndex )
	{
		dataIndex = inAryHdl->dataNum;
	}

	// 挿入位置以降のレコードをずらす
	if( dataIndex < inAryHdl->dataNum )
	{ // ずらす必要がある
		newPos = GetDataPosByIndex( inAryHdl->recPtr, inAryHdl->recSize, dataIndex + 1 );  // 移動後の位置
		oldPos = GetDataPosByIndex( inAryHdl->recPtr, inAryHdl->recSize, dataIndex );      // 移動前の位置
		shiftDataSize = inAryHdl->recSize * ( inAryHdl->dataNum - dataIndex );  // ずらすデータのサイズ
		memmove( newPos, oldPos, shiftDataSize );  // 領域が重なっている可能性のある移動
	}

	// データを挿入
	newDataPos = GetDataPosByIndex( inAryHdl->recPtr, inAryHdl->recSize, dataIndex );
	if( inInsertRecPtr != NULL )
	{
		// データを書き込む
		memcpy( newDataPos, inInsertRecPtr, inAryHdl->recSize );
	}
	else
	{
		// 0クリアされたデータを書き込む
		memset( newDataPos, 0, inAryHdl->recSize );
	}

	// データ数を更新する
	inAryHdl->dataNum = inAryHdl->dataNum + 1;

	return 0;  // データの挿入に成功
}

// -------------------------------------------------
// 関数 : Jiro_Ary_Add
// 機能 : レコード（データ）を追加
// -------------------------------------------------
int Jiro_Ary_Add( JIRO_ARY_HDL inAryHdl, void* inAddRecPtr )
{
	int rc;

	// データを後ろに追加
	rc = Jiro_Ary_Insert( inAryHdl, inAryHdl->dataNum + 1, inAddRecPtr );

	return rc;
}

// -------------------------------------------------
// 関数 : Jiro_Ary_Update
// 機能 : レコード（データ）を更新
// -------------------------------------------------
void Jiro_Ary_Update( JIRO_ARY_HDL inAryHdl, long inIndex, void* inRecPtr )
{
	void* dataPos;

	// ＮＵＬＬチェック
	if( inAryHdl == NULL )
	{
	     return;
	}
	if( inAryHdl->recPtr == NULL )
	{
	     return;
	}

	// インデックスチェック
	if( IsValidIndex( inAryHdl, inIndex ) == false )
	{
	     return;
	}

	// 値を更新
	dataPos = GetDataPosByIndex( inAryHdl->recPtr, inAryHdl->recSize, inIndex );
	memcpy( dataPos, inRecPtr, inAryHdl->recSize );
}

//--------------------------------------------------
// 関数 : IsValidIndex
// 機能 : 現在の配列に対して使用可能なインデックスかどうか
// 返値 : true  : 使用可能(範囲内)
//        false : 使用不可(範囲外)
//--------------------------------------------------
static bool IsValidIndex( JIRO_ARY_HDL inAryHdl, long inIndex )
{
     // 0未満だとダメ
     if( inIndex < 0 )
     {
	  return false;
     }

     // データ数以上だとダメ
     if( inAryHdl->dataNum <= inIndex )
     {
	  return false;
     }

     return true;
}

//--------------------------------------------------
// 関数 : GetDataPosByIndex
// 機能 : 指定されたインデックスに対応するデータの位置
//--------------------------------------------------
static void* GetDataPosByIndex( void* inRecPtr, long inRecSize, long inIndex )
{
	char* basePos;
	char* dataPos;

	// 基準位置
	basePos = (char*)inRecPtr;

	// データ位置
	dataPos = basePos + ( inRecSize * inIndex );

	return (void*)dataPos;
}

// -------------------------------------------------
// 関数 : Jiro_Ary_Remove
// 機能 : レコード（データ）を削除
// -------------------------------------------------
void Jiro_Ary_Remove( JIRO_ARY_HDL inAryHdl, long inRemoveIndex )
{
	void* specifiedDataPos;
	void* nextDataPos;
	long moveDataSize;

	// ＮＵＬＬチェック
	if( inAryHdl == NULL )
	{
		return;
	}
	if( inAryHdl->recPtr == NULL )
	{
		return;
	}

	// インデックスチェック
	if( IsValidIndex( inAryHdl, inRemoveIndex ) == false )
	{
		return;
	}

	// ずらすデータのサイズ
	moveDataSize = inAryHdl->recSize * ( inAryHdl->dataNum - inRemoveIndex - 1 );

	// ずらす必要があるか
	if( moveDataSize <= 0 )
	{
		inAryHdl->dataNum = inAryHdl->dataNum - 1;	// データ数を更新
		return;
	}

	// 指定されたデータの位置
	specifiedDataPos = GetDataPosByIndex( inAryHdl->recPtr, inAryHdl->recSize, inRemoveIndex );

	// 指定されたデータの次のデータの位置
	nextDataPos = GetDataPosByIndex( inAryHdl->recPtr, inAryHdl->recSize, inRemoveIndex + 1 );

	// 指定されたデータより後のデータを、指定されたデータの位置にずらす
	memmove( specifiedDataPos, nextDataPos, moveDataSize );  // memmoveでは領域が重なっていてもよい。
	inAryHdl->dataNum = inAryHdl->dataNum - 1;	// データ数を更新
}

//--------------------------------------------------
// 関数 : Jiro_Ary_RemoveAll
// 機能 : 動的配列すべてのレコードを削除
//--------------------------------------------------
void Jiro_Ary_RemoveAll( JIRO_ARY_HDL inAryHdl )
{
     // 配列管理テーブルを更新
     inAryHdl->recNum = 0;
     inAryHdl->dataNum = 0;

     // データ領域を解放
	 if( inAryHdl->recPtr != NULL )
	 {
		free( inAryHdl->recPtr );
		inAryHdl->recPtr = NULL;
	 }
}

//--------------------------------------------------
// 関数 : Jiro_Ary_Swap
// 機能 : 動的配列レコードの順番を入れ替える
//--------------------------------------------------
void Jiro_Ary_Swap( JIRO_ARY_HDL inAryHdl, long inIndex1, long inIndex2 )
{
	void* data1Pos;
	void* data2Pos;
	void* tempData;

	// ＮＵＬＬチェック
	if( inAryHdl == NULL )
	{
		return;
	}
	if( inAryHdl->recPtr == NULL )
	{
		return;
	}

	// インデックスチェック
	if( IsValidIndex( inAryHdl, inIndex1 ) == false )
	{
		return;
	}
	if( IsValidIndex( inAryHdl, inIndex2 ) == false )
	{
		return;
	}

	// データの位置を取得
	data1Pos = GetDataPosByIndex( inAryHdl->recPtr, inAryHdl->recSize, inIndex1 );
	data2Pos = GetDataPosByIndex( inAryHdl->recPtr, inAryHdl->recSize, inIndex2 );

	// データの一時置き場を取得
	tempData = malloc( inAryHdl->recSize );
	if( tempData == NULL )
	{
		return;
	}

	// データを入れ替え
	memcpy(  tempData, data1Pos, inAryHdl->recSize );
	memmove( data1Pos, data2Pos, inAryHdl->recSize );  // memo tamura 081205 memmove なのは inIndex1 == inIndex2 の場合のため？
	memcpy(  data2Pos, tempData, inAryHdl->recSize );

	// 一時置き場を解放
	free( tempData );
}

//--------------------------------------------------
// 関数 : Jiro_Ary_QuickSort
// 機能 : 動的配列をクイックソートで並び替える
//--------------------------------------------------
void Jiro_Ary_QuickSort( JIRO_ARY_HDL inAryHdl, QUICKSORT_COMPARE_FUNC_PTR inCompareFuncPtr )
{
	// ＮＵＬＬチェック
	if( inAryHdl == NULL )
	{
		return;
	}
	if( inAryHdl->recPtr == NULL )
	{
		return;
	}

	// ソートするデータがあるか
	if( inAryHdl->dataNum < 2 )
	{
		return;  // ソートの必要なし
	}

	// クイックソート
	qsort( inAryHdl->recPtr, inAryHdl->dataNum, inAryHdl->recSize, inCompareFuncPtr );
}

//--------------------------------------------------
// 関数 : Jiro_Ary_GetDataPtr
// 機能 : 動的配列レコードへのポインタを取得
//--------------------------------------------------
void* Jiro_Ary_GetDataPtr( JIRO_ARY_HDL inAryHdl, long inIndex )
{
	void* dataPos;

	if( inAryHdl->recPtr == NULL )
	{ // データがまだない
		return NULL;
	}
	if( IsValidIndex( inAryHdl, inIndex ) == false )
	{ // インデックスが範囲外
		return NULL;
	}

	// データ位置
	dataPos = GetDataPosByIndex( inAryHdl->recPtr, inAryHdl->recSize, inIndex );

	return dataPos;
}
