

// TYPE: C
// PRJ: JCL
// PGM: Jiro_Array
// VER: 2.00
// USES: GCC
// CREATE: 2002/09/02( Shiratori )
// UPDATE: 2007/04/02( tamura )
/////////////////////////////////////////////////////
// ＯＳ依存関数をなくし、ＡＮＳＩ準拠関数で作成。
// バージョン１に比べ、メモリ確保速度なども向上している。
#ifndef __JIRO_ARRAY__
#define __JIRO_ARRAY__

#ifdef __cplusplus
extern "C" {
#endif

//*************************************************************************
// 定数
//*************************************************************************
#define kDefaultInitRecNum   20
#define kDefaultExpandRecNum 20

//*************************************************************************
// 構造体
//*************************************************************************
// 配列管理テーブル
typedef struct
{
  void*		recPtr;       // データ領域バッファへのポインタ
  long		recNum;        // 領域確保済レコード数
  long		dataNum;       // 使用中レコード数
  long		recSize;       // 1レコードのサイズ
  long		initRecNum;    // 初期レコード数
  long		expandRecNum;  // 増分レコード数
} JIRO_ARY;

typedef JIRO_ARY* JIRO_ARY_HDL;
typedef int( * QUICKSORT_COMPARE_FUNC_PTR )( const void *, const void * );

//*************************************************************************
// 関数
//*************************************************************************
// -------------------------------------------------
// 関数 : Jiro_Ary_Init
// 機能 : 動的配列を初期化
// -------------------------------------------------
JIRO_ARY_HDL Jiro_Ary_Init( long inRecSize, long initNumOfRecs, long inExpandNumOfRecs );

// -------------------------------------------------
// 関数 : Jiro_Ary_Free
// 機能 : 動的配列を解放
// -------------------------------------------------
void Jiro_Ary_Free( JIRO_ARY_HDL inAryHdl );

// -------------------------------------------------
// 関数 : Jiro_Ary_GetSize
// 機能 : レコード（データ）数を取得
// -------------------------------------------------
long Jiro_Ary_GetSize( JIRO_ARY_HDL inAryHdl );

// -------------------------------------------------
// 関数 : Jiro_Ary_GetData
// 機能 : レコード（データ）を取得
// -------------------------------------------------
void Jiro_Ary_GetData( JIRO_ARY_HDL inAryHdl, long inIndex, void* outRecPtr );

// -------------------------------------------------
// 関数 : Jiro_Ary_Insert
// 機能 : レコード（データ）を挿入
// 戻り値(0: 成功、-1: メモリ不足エラー)
// -------------------------------------------------
int Jiro_Ary_Insert( JIRO_ARY_HDL inAryHdl, long inIndex, void* inInsertRecPtr );

// -------------------------------------------------
// 関数 : Jiro_Ary_Add
// 機能 : レコード（データ）を追加
// -------------------------------------------------
int Jiro_Ary_Add( JIRO_ARY_HDL inAryHdl, void* inAddRecPtr );

// -------------------------------------------------
// 関数 : Jiro_Ary_Update
// 機能 : レコード（データ）を更新
// -------------------------------------------------
void Jiro_Ary_Update( JIRO_ARY_HDL inAryHdl, long inIndex, void* inRecPtr );

// -------------------------------------------------
// 関数 : Jiro_Ary_Remove
// 機能 : レコード（データ）を削除
// -------------------------------------------------
void Jiro_Ary_Remove( JIRO_ARY_HDL inAryHdl, long inRemoveIndex );

//--------------------------------------------------
// 関数 : Jiro_Ary_RemoveAll
// 機能 : 動的配列すべてのレコード（データ）を削除
//--------------------------------------------------
void Jiro_Ary_RemoveAll( JIRO_ARY_HDL inAryHdl );

//--------------------------------------------------
// 関数 : Jiro_Ary_Swap
// 機能 : 動的配列レコードの順番を入れ替える
//--------------------------------------------------
void Jiro_Ary_Swap( JIRO_ARY_HDL inAryHdl, long inIndex1, long inIndex2 );

//--------------------------------------------------
// 関数 : Jiro_Ary_QuickSort
// 機能 : 動的配列をクイックソートで並び替える
//--------------------------------------------------
void Jiro_Ary_QuickSort( JIRO_ARY_HDL inAryHdl, QUICKSORT_COMPARE_FUNC_PTR inCompareFuncPtr );

//--------------------------------------------------
// 関数 : Jiro_Ary_GetDataPtr
// 機能 : 動的配列レコードへのポインタを取得
//--------------------------------------------------
void* Jiro_Ary_GetDataPtr( JIRO_ARY_HDL inAryHdl, long inIndex );

#ifdef __cplusplus
}
#endif

#endif // __JIRO_ARRAY__
