// TYPE: C
// PRJ: JCL
// PGM: Jiro_String
// VER: 2.00
// USES: CodeWarrior, Xcode, Visual C++ 6.0-8.0, JCL
// CREATE: 1999/02/21( Siratori )
// UPDATE: 1999/10/01( Siratori )
// UPDATE: 2003/04/09( tsunoda )
// UPDATE: 2005/03/15( hisa )
// UPDATE: 2007/02/20( hisa, igari )
// UPDATE: 2007/08/28( tamura )
// UPDATE: 2008/06/06( tamura, sugimoto )
// UPDATE: 2009/05/13( hisa )
// UPDATE: 2009/08/18( hisa )
// UPDATE: 2009/07/30( hisa )
/////////////////////////////////////////////////////
// MFCプロジェクトを使用する場合は、下記のコメントを外す。
//#include "stdafx.h"

// ansi
#include <string.h>
#include <ctype.h>
#include <stdio.h>
#include <math.h>
#include <stdlib.h>

// JCL
#include "Jiro_Const.h"
#include "Jiro_String.h"
#include "Jiro_File.h"

//*************************************************************************
// 内部関数のプロトタイプ宣言
//*************************************************************************
static long GetEncoding( const char* inTextData, long inTextDataSize );
static long CheckSJIS( const char* inTextData, long inTextDataSize );
static long CheckEUC( const char* inTextData, long inTextDataSize );
static long CheckUTF8( const char* inTextData, long inTextDataSize );
static bool IsASCII( const char* inTextData, long inTextDataSize );
static bool IsISO2022JP( const char* inTextData, long inTextDataSize );

//*************************************************************************
// 関数
//*************************************************************************
//-------------------------------------------------------------
// 関数 : Jiro_Pstrcpy
// 機能 : パスカル文字列コピー
//-------------------------------------------------------------
void Jiro_Pstrcpy( unsigned char* outDestStr, unsigned char* inSrcStr )
{
	short	len;
	
	if (!outDestStr || !inSrcStr)
	{
		return;
	}
	
	len = inSrcStr[0];
	memcpy( outDestStr, inSrcStr, len + 1 );
}

//-------------------------------------------------------------
// 関数 : Jiro_Pstrcat
// 機能 : パスカル文字列連結
//-------------------------------------------------------------
void Jiro_Pstrcat( unsigned char* ioStr, unsigned char* inAddStr )
{
	short	srcLen, addLen;
	
	if (!ioStr || !inAddStr )
	{
		return;
	}
	
	srcLen = ioStr[0];
	addLen = inAddStr[0];
	
	if (addLen + srcLen > kJIRO_UCHAR_MAX)
	{
		addLen = kJIRO_UCHAR_MAX - srcLen;
	}
	
	memcpy( &ioStr[srcLen + 1], &inAddStr[1] , addLen );
	ioStr[0] += ( (unsigned char)addLen );
}

//-------------------------------------------------------------
// 関数 : Jiro_Cstr2Pstr
// 機能 : Ｃ文字列をパスカル文字列に変換
//-------------------------------------------------------------
void Jiro_Cstr2Pstr( unsigned char* outDestStr, char* inSrcStr )
{
	long	len;
	
	if (!outDestStr || !inSrcStr)
	{
		return;
	}
	
	len = (long)strlen(inSrcStr);
	if (len > kJIRO_UCHAR_MAX)
	{
		len = kJIRO_UCHAR_MAX;
	}
	
	memcpy(&outDestStr[1], inSrcStr, len);
	outDestStr[0] = (unsigned char)len;
}

//-------------------------------------------------------------
// 関数 : Jiro_Pstr2Cstr
// 機能 : パスカル文字列をＣ文字列に変換
//-------------------------------------------------------------
void Jiro_Pstr2Cstr( char* outDestStr, unsigned char* inSrcStr )
{
	short	len;
	
	if (!outDestStr || !inSrcStr)
	{
		return;
	}
	
	len = inSrcStr[0];
	memcpy( outDestStr, &inSrcStr[1], len );
	outDestStr[len] = '\0';
}

//-------------------------------------------------------------
// 関数 : Jiro_CutStringLength
// 機能 : 文字列長を指定された長さでカット
//-------------------------------------------------------------
void Jiro_CutStringLength( char* ioStr, long inLength )
{
	short	attr;
	long	len;
	
	if (!ioStr)
	{
		return ;
	}
	
	len = (long)strlen( ioStr );
	if ( len < inLength )	// 文字列長を超える？
	{
		return ;
	}
	
	if ( inLength <= 0 )			// 長さ０を指定？
	{
		ioStr[0] = '\0';
		return ;
	}
	
	attr = Jiro_GetCharAttr((unsigned char*) ioStr, inLength - 1);	// 文字属性取得
	if ((attr == 0) || (attr == 2))		// 半角または全角の２バイト目？
	{
		ioStr[inLength] = '\0';
	}
	else
	{
		ioStr[inLength - 1] = '\0';
	}
}

//-------------------------------------------------------------
// 関数 : Jiro_GetCharAttr
// 機能 : 文字の属性を取得
// 戻り値： 0：半角、１：全角の１バイト目、２：全角の２バイト目
// 1999.10.1-1 add
//-------------------------------------------------------------
short Jiro_GetCharAttr( unsigned char* ioStr, long nOff )
{
	bool	bKanji = false; //  mod takaesu 070730
	short	nAtr;
	long	i;
	
	i = 0;
	while (i <= nOff)
	{
		if (Jiro_IsZenkaku1(ioStr[i]))
		{
			bKanji = true;
			i += 2;
		}
		else
		{
			bKanji = false;
			i++;
		}
	}
	if (i >= (nOff + 2))
	{
		nAtr = 1;
	}
	else
	{
		if (bKanji)
		{
			nAtr = 2;
		}
		else
		{
			nAtr = 0;
		}
	}
	return (nAtr);
}

//-------------------------------------------------------------
// 関数 : Jiro_IsZenkaku1
// 機能 : 全角文字の１バイト目？
//-------------------------------------------------------------
bool Jiro_IsZenkaku1( unsigned char inCh )
{
	return ((inCh >= 0x81 && inCh <= 0x9F) || (inCh >= 0xE0 && inCh <= 0xFC)) ? true : false;
}

//-------------------------------------------------------------
// 関数 : Jiro_IsZenkaku2
// 機能 : 全角文字の２バイト目？
// 戻値： true（全角文字の２バイト目）、false（全角文字の２バイト目ではない）
//-------------------------------------------------------------
bool Jiro_IsZenkaku2( unsigned char inCh )
{
	return ((inCh >= 0x40) && (inCh <= 0xFC) && (inCh != 0x7F)) ? true : false;
}

//-------------------------------------------------------------
// 関数 : Jiro_IsNumberStr
// 機能 : 数値文字列？
// 戻値 : true（数値文字列）、false（数値文字列ではない）
//-------------------------------------------------------------
bool Jiro_IsNumberStr( char* inStr )
{
	while( *inStr )
	{
		if ( (isdigit(*inStr) == 0) && (*inStr != '-') && (*inStr != '+'))
		{
			return false;
		}
		inStr++;
	}
	return true;
}

//-------------------------------------------------------------
// 関数 : Jiro_IsNumOrPeriodStr
// 機能 : 数値またはピリオドの文字列？
// 戻値 : true（数値文字列）、false（数値文字列ではない）
//-------------------------------------------------------------
bool Jiro_IsNumOrPeriodStr( char* inStr )
{
	while( *inStr )
	{
		if ((isdigit(*inStr) == 0) && (*inStr != '.') && (*inStr != '-') && (*inStr != '+'))
		{
			return false;
		}
		inStr++;
	}
	return true;
}

//-------------------------------------------------------------
// 関数 : Jiro_LongToCommaString
// 機能 : 数値をコンマ付の文字列に変換
//-------------------------------------------------------------
void Jiro_LongToCommaString( long inNum, char* outDestStr )
{
	char	numStr[50];
	
	sprintf( numStr, "%ld", inNum );
	Jiro_NumStrToCommaString( numStr, outDestStr );
}

//-------------------------------------------------------------
// 関数 : Jiro_NumStrToCommaStr
// 機能 : 数値文字列をコンマ付の文字列に変換
//-------------------------------------------------------------
void Jiro_NumStrToCommaString( char* inSrcStr, char* outDestStr )
{
	char	intStr[256];
	long	len, nInt,workPos;
	char*	periodPos;
	char	periodStr[256];
	char 	siginStr[10];
	
	if (inSrcStr[0] == '\0')
	{
		outDestStr[0] = '\0';
		return;
	}
	if (Jiro_IsNumOrPeriodStr(inSrcStr) == false)	// 数字とコンマ以外の文字がある？
	{
		strcpy( outDestStr, inSrcStr );	// そのまま帰す
		return;
	}
	
	// 小数点以下の文字列は、待避させておく。
	periodStr[0] = '\0';
	periodPos = strrchr(inSrcStr,'.');
	if (periodPos)
	{
		*periodPos = '\0';
		strcpy(periodStr, periodPos + 1);
	}
	
	// 符号文字は、待避させておく
	siginStr[0] = '\0';
	if ((inSrcStr[0] == '+') || (inSrcStr[0] == '-'))
	{
		siginStr[0] = inSrcStr[0];
		siginStr[1] = '\0';
		inSrcStr++;
	}
	
	// 整数部にコンマを付けた文字列を作成
	len = (long)strlen(inSrcStr);
	if (!len)
	{
		strcpy(intStr, "0");
	}
	else
	{
		workPos = len + (len-1) / 3;
		intStr[workPos--] = '\0';
		nInt = 0;
		while ((--len) >= 0)
		{
			if ((nInt % 3 == 0) && nInt)
			{
				intStr[workPos--] = ',';
			}
			intStr[workPos--] = inSrcStr[len];
			nInt++;
		}
	}
	if ((long)strlen(periodStr) >= 1)
	{
		sprintf( outDestStr, "%s%s.%s", siginStr, intStr, periodStr );
	}
	else
	{
		sprintf( outDestStr, "%s%s", siginStr, intStr );
	}
}

//-------------------------------------------------------------
// 関数 : Jiro_StripCharFromEnd
// 機能 : 文字列の後ろにある、指定文字を削除
//-------------------------------------------------------------
void Jiro_StripCharFromEnd( char* ioStr, char inDelCh )
{
	char* pos;
	
	if (*ioStr == '\0')
	{
		return;
	}
	
	pos = ioStr + (long)strlen(ioStr) - 1;
	while ((*pos == inDelCh) && (pos >= ioStr))
	{
		pos--;
	}
	
	*(pos + 1) = '\0';
}

//-------------------------------------------------------------
// 関数 : Jiro_StripCharFromStart
// 機能 : 文字列の先頭にある、指定文字を削除
//-------------------------------------------------------------
void Jiro_StripCharFromStart( char* ioStr, char inDelCh )
{
	long	len;
	long	delCnt;
	char*	pos;
	
	if (*ioStr == '\0')
	{
		return;
	}
	
	len = (long)strlen( ioStr );
	delCnt = 0;
	pos = ioStr;
	while( *pos == inDelCh )
	{
		delCnt++;
		pos++;
	}
	memmove( ioStr, ioStr + delCnt, len - delCnt + 1 );
}

//-------------------------------------------------------------
// 関数 : Jiro_ReplaceString
// 機能 : 文字列の置換処理
//-------------------------------------------------------------
void Jiro_ReplaceString( char* ioTargetStr, const char* inFindStr, const char* inReplaceStr )
{
	long	findLen;
	long	replaceLen;
	char*	targetPos;
	char*	foundPos;
	char*	moveSrcPos;
	
	if (ioTargetStr == NULL)
	{
		return;
	}
	if (ioTargetStr[0] == '\0')
	{
		return;
	}
	if (inFindStr == NULL)
	{
		return;
	}
	if (inFindStr[0] == '\0')
	{
		return;
	}
	
	findLen = (long)strlen(inFindStr);
	if (inReplaceStr == NULL)
	{
		replaceLen = 0;
	}
	else
	{
		replaceLen = (long)strlen(inReplaceStr);
	}
	
	targetPos = ioTargetStr;
	foundPos = strstr(targetPos, inFindStr);
	while (foundPos)
	{
		moveSrcPos = foundPos + findLen;
		memmove(foundPos + replaceLen, moveSrcPos, (long)strlen(moveSrcPos) + 1);
		if (inReplaceStr)
		{
			memcpy(foundPos, inReplaceStr, replaceLen);
		}
		targetPos = foundPos + replaceLen;
		
		foundPos = strstr(targetPos, inFindStr);
	}
}

//-------------------------------------------------------------
// 関数 : Jiro_Num_To_2byteStr
// 機能 : 数字を全角文字列に変換
//-------------------------------------------------------------
void Jiro_Num_To_2byteStr( long inNum, char* outStr )
{
	char	numStr[20];
	long	i;
	long	len;
	
	sprintf(numStr, "%ld", inNum);
	len = (long)strlen(numStr);
	for (i = 0; i < len; i++)
	{
		outStr[i * 2] = (char)0x82;
		outStr[(i * 2) + 1] = 0x4F + (numStr[i] - 0x30);
	}
	outStr[len * 2] = '\0';
}


//-------------------------------------------------------------
// 関数 : Jiro_GetIndexOfLastChar
// 機能 : 全角の混じった文字列から指定した半角文字が最後に出てくるインデックスを返す
// add_tsunoda 030409
//-------------------------------------------------------------
long Jiro_GetIndexOfLastChar( const char* inStr, char inOffset )
{
	long index;
	
	index = (long)strlen( inStr );
	
	if( inStr == NULL )
	{
		return 0;
	}
	
	while( index >= 0 && 
		  !( inStr[index] == inOffset && Jiro_GetCharAttr(( unsigned char*)inStr,index) == 0 ) )
	{
		index--;
	}
	return index;
}

//-------------------------------------------------------------
// 関数 : Jiro_IsTheSamedDouble
// 機能 : ２つのＤＯＵＢＬＥ型が一致？
// add_hisa 050315
//-------------------------------------------------------------
bool Jiro_IsTheSamedDouble( double inVal1, double inVal2 )
{
	long	len;
	long	val1, val2;
	
	// 精度を合わせてから引く
	val1 = (long)( inVal1 * 1000000 );
	val2 = (long)( inVal2 * 1000000 );
	len = val1 - val2;
	if( len == 0 )
	{
		return true;	// 一致
	}
	
	return false;	// 不一致
}

//-------------------------------------------------------------
// 関数 : Jiro_DoubleToChar
// 機能 : ＤＯＵＢＬＥ型を文字列型に変換( 小数点以下を考慮 )
// add_hisa 050315
//-------------------------------------------------------------
void Jiro_DoubleToChar( double inNumVal, char* outNumBuff )
{
	double	decimalVal;		// 小数部
	double	integerVal;		// 整数部
	double	multipleVal;	// 10倍数
	
	// 小数点なし
	integerVal = floor( inNumVal );
	decimalVal = inNumVal - integerVal;	
	if( Jiro_IsTheSamedDouble( decimalVal, 0.000000 ) == true )
	{
		sprintf( outNumBuff, "%.0f", inNumVal );
		return;	// 小数点なし
	}
	
	// 小数点第一位
	multipleVal = inNumVal * 10;
	integerVal = floor( multipleVal );	
	decimalVal = multipleVal - integerVal;
	if( Jiro_IsTheSamedDouble( decimalVal, 0.000000 ) == true )
	{
		sprintf( outNumBuff, "%.1f", inNumVal );
		return;	// 小数点第一位まで
	}
	
	// 小数点第二位
	multipleVal = inNumVal * 100;
	integerVal = floor( multipleVal );	
	decimalVal = multipleVal - integerVal;
	if( Jiro_IsTheSamedDouble( decimalVal, 0.000000 ) == true )
	{
		sprintf( outNumBuff, "%.2f", inNumVal );
		return;	// 小数点第二位まで
	}
	
	// 小数点第三位
	multipleVal = inNumVal * 1000;
	integerVal = floor( multipleVal );	
	decimalVal = multipleVal - integerVal;
	if( Jiro_IsTheSamedDouble( decimalVal, 0.000000 ) == true )
	{
		sprintf( outNumBuff, "%.3f", inNumVal );
		return;	// 小数点第三位まで
	}
	
	// 小数点第四位
	multipleVal = inNumVal * 10000;
	integerVal = floor( multipleVal );	
	decimalVal = multipleVal - integerVal;
	if( Jiro_IsTheSamedDouble( decimalVal, 0.000000 ) == true )
	{
		sprintf( outNumBuff, "%.4f", inNumVal );
		return;	// 小数点第四位まで
	}
	
	// 小数点第五位
	multipleVal = inNumVal * 100000;
	integerVal = floor( multipleVal );	
	decimalVal = multipleVal - integerVal;
	if( Jiro_IsTheSamedDouble( decimalVal, 0.000000 ) == true )
	{
		sprintf( outNumBuff, "%.5f", inNumVal );
		return;	// 小数点第五位まで
	}
	
	// 小数点第六位
	sprintf( outNumBuff, "%.6f", inNumVal );
}

//-------------------------------------------------------------
// 関数 : Jiro_Malloc
// 機能 : バッファを確保
// add_hisa 050315
//-------------------------------------------------------------
char* Jiro_Malloc( long inBuffSize )
{
	char*	outBuffer;
	
	outBuffer = ( char* )malloc( inBuffSize );
	if( outBuffer == NULL )
	{
		return NULL;
	}
	memset( outBuffer, 0, inBuffSize );
	
	return outBuffer;
}

//-------------------------------------------------------------
// 関数 : Jiro_MallocFree
// 機能 : バッファを解放
// add_hisa 050315
//-------------------------------------------------------------
void Jiro_MallocFree( void* inBuffer )
{
	if( inBuffer == NULL )
	{
		return;
	}
	free( inBuffer );
}

//-------------------------------------------------------------
// 関数： Jiro_GetViewDateStringFromDataChars
// 機能 : 日時を文字列に
// char inSourceChars[18]  // YYYYMMDDHHMMSS000
// char outDestStr[17]  // YYYY/MM/DD HH:MM
//			               01234567890123456
// add_igari 070220
//-------------------------------------------------------------
void Jiro_GetViewDateStringFromDataChars( char* inSourceChars, char* outDestStr )
{
	sprintf( outDestStr, "YYYY/MM/DD HH:MM" );
	memcpy( &(outDestStr[ 0]), &(inSourceChars[ 0]), 4 ); // YYYY
	memcpy( &(outDestStr[ 5]), &(inSourceChars[ 4]), 2 ); // MM
	memcpy( &(outDestStr[ 8]), &(inSourceChars[ 6]), 2 ); // DD
	memcpy( &(outDestStr[11]), &(inSourceChars[ 8]), 2 ); // HH
	memcpy( &(outDestStr[14]), &(inSourceChars[10]), 2 ); // MM
}

//-------------------------------------------------------------
// 関数： Jiro_GetFileSizeString
// 機能 : ファイルサイズ文字列を取得
// add_igari 070220
//-------------------------------------------------------------
void Jiro_GetFileSizeString( long inFileSize, char* outFileSizeStr )
{
	double	sizeMB;
	double	sizeKB;
	
	// MBに変換
	sizeMB = (double)inFileSize / 1024.0 / 1024.0;
	
	// 1MB以上？
	if( 1.0 <= sizeMB )
	{
		// ファイルサイズをMB単位、小数点第一位にする
		sprintf( outFileSizeStr, "%.1f MB", sizeMB );		
		return;
	}
	
	// 1MB未満なら、KB単位、少数第一位にする
	sizeKB = (double)inFileSize / 1024.0;	
	if( sizeKB <= 1.0 )
	{
		sizeKB = 0.000000;
	}
	sprintf( outFileSizeStr, "%.1f KB", sizeKB );
}

//-------------------------------------------------------------
// 関数 : Jiro_MakeVersionString 
// 機能 : バージョン文字列を作成
// inComma = カンマ数（０、１，２）、それ以外はすべての桁
// add_hisa 070220
//-------------------------------------------------------------
void Jiro_MakeVersionString( const char* inValue, long inComma, char* outVersion )
{
	char	verStr1[8] = {0};
	long	ver1;
	char	verStr4[8] = {0};
	long	ver4;
	
	sprintf( verStr1, "%c%c", inValue[0], inValue[1] );
	ver1 = atol( verStr1 );
	
	// カンマなしバージョン
	if( inComma == 0 )
	{
		sprintf( outVersion, "%ld", ver1 );
	}
	// カンマ１バージョン
	else if( inComma == 1 )
	{
		sprintf( outVersion, "%ld.%c", ver1, inValue[2] );
	}
	// カンマ２バージョン
	else if( inComma == 2 )
	{
		sprintf( outVersion, "%ld.%c.%c", ver1, inValue[2], inValue[3] );
	}
	// カンマ３バージョン
	else
	{
		sprintf( verStr4, "%c%c%c", inValue[4], inValue[5], inValue[6] );
		ver4 = atol( verStr4 );	
		sprintf( outVersion, "%ld.%c.%c (%ld)", ver1, inValue[2], inValue[3], ver4 );
	}
}

//-------------------------------------------------------------
// 関数 : Jiro_Strcmp
// 機能 : 大文字小文字を区別せずに文字列を比較する
// メモ : strcasecmpがwindowsにないので、自前で実装
// add tamura 070828
//-------------------------------------------------------------
long Jiro_Strcmp( const char* s1, const char* s2 )
{
	long i;
	
	// 先頭からたどっていく
	i = 0;
	while( s1[ i ] != '\0' )
	{
		// 大文字にそろえてから比較する
		if( toupper( (unsigned char)s1[ i ] ) != toupper( (unsigned char)s2[ i ] ) )  // s[2]がヌル文字になったらここで引っかかる
		{ // 違った!
			return toupper( (unsigned char)s1[ i ] ) - toupper( (unsigned char)s2[ i ] );
		}
		
		// 次の文字に進む
		i++;
	}
	// s1の最後まで見た
	
	// s2がまだ残ってるかもしれないので、ヌル文字(s1[i])と比較する
	return toupper( (unsigned char)s1[ i ] ) - toupper( (unsigned char)s2[ i ] );
}

//-------------------------------------------------------------
// 関数 : Jiro_ConvertSecToDateString
// 機能 : 秒数を日付に変換する
// sugimoto 080606
//-------------------------------------------------------------
void Jiro_ConvertSecToDateString( unsigned long inSec, char* outTimeStr )
{
	long i;
	long day;
	long month;
	long year;
	long monthData[12];
	long daysByYear = 365;
	long uruudosi = 0;
	
	// 初期化
	year = 2000;
	// 秒数を日数に変更（０日を１日にするために切り上げ）
	day = inSec / 86400 + 1;		// １日は８６４００秒	
	// 日数から年の抽出
	while( 0 < day )
	{
		if( (year%4 == 0 && year%100 != 0) || year%400 == 0 )
		{
			// 閏年だった
			daysByYear= 366;
			uruudosi = 1;
		}
		else
		{
			// 平年だった
			daysByYear= 365;
			uruudosi = 0;
		}
		if( day - daysByYear > 0 )
		{
			// 次の年へ	
			year++;
			day -= daysByYear;
		}
		else
		{
			// 年決定
			break;
		}
	}
	
	// 残った日数データはその年の１月１日からの日数になる
	// 月データ作成
	monthData[0] = 31;					// １月
	monthData[1] = 28 + uruudosi; 		// ２月
	monthData[2] = 31; 					// ３月
	monthData[3] = 30; 					// ４月
	monthData[4] = 31; 					// ５月
	monthData[5] = 30; 					// ６月
	monthData[6] = 31; 					// ７月
	monthData[7] = 31; 					// ８月
	monthData[8] = 30; 					// ９月
	monthData[9] = 31; 					// １０月
	monthData[10] = 30; 				// １１月
	monthData[11] = 31; 				// １２月
	month = 0;
	for( i=0; i<12; i++ )
	{
		if( day - monthData[i] > 0 )
		{
			// 月ごとに日数を減らす
			day -= monthData[i];
		}
		else
		{
			// 月が決定
			month = i+1;
			break;
		}
	}
	
	sprintf( outTimeStr, "%ld月%ld日", month, day );
}

//-------------------------------------------------------------
// 関数 : Jiro_rindex
// 機能 : 全角の混じった文字列から指定した半角文字が最後に出てくる位置を返す
// シフトＪＩＳ限定
// add tamura 080606
//-------------------------------------------------------------
char* Jiro_rindex( const char* inStr, char inChar )
{
	long lastIndex;
	long i;
	
	// 引数チェック
	if( inStr == NULL )
	{
		return NULL;
	}
	
	// 文字列の最後の位置を取得
	lastIndex = ( long )strlen( inStr );
	
	// 文字列を後ろから見て指定の文字を探す
	for( i = lastIndex;  0 <= i;  i-- )
	{
		// 指定の文字か
		if( ( inStr[ i ] == inChar ) &&
		   ( Jiro_GetCharAttr( ( unsigned char*)inStr, i ) == 0 ) // 半角文字か
		   )
		{
			// 見つかった文字の位置を返す
			return ( char* ) &( inStr[ i ] );
		}
	}
	
	// 見つからなかった
	return NULL;
}

//-------------------------------------------------------------
// 関数 : Jiro_index
// 機能 : 全角の混じった文字列から指定した半角文字が最初に出てくる位置を返す
// シフトＪＩＳ限定
// add tamura 080606
//-------------------------------------------------------------
char* Jiro_index( const char* inStr, char inChar )
{
	long lastIndex;
	long i;
	
	// 引数チェック
	if( inStr == NULL )
	{
		return NULL;
	}
	
	// 文字列サイズを取得
	lastIndex = ( long )strlen( inStr );
	
	// 文字列を前から見て指定の文字を探す
	for( i=0; i<lastIndex; i++ )
	{
		// 指定の文字か
		if( ( inStr[ i ] == inChar ) &&
		   ( Jiro_GetCharAttr( ( unsigned char*)inStr, i ) == 0 ) )	// 半角文字か
		{
			// 見つかった文字の位置を返す
			return ( char* ) &( inStr[ i ] );
		}
	}
	
	// 見つからなかった
	return NULL;
}

//-------------------------------------------------------------
// 関数 : Jiro_Strcasestr
// 機能 : 部分文字列が文字列中で最初に現れる位置を見つける（両方の引数に対して大文字小文字を無視）
// add tamura 080606
//-------------------------------------------------------------
const char* Jiro_Strcasestr( const char* inHaystack, const char* inNeedle )
{
	const char*	pos;
	const char*	startn = NULL;
	const char*	nextPos = NULL;
	
	for ( pos = inHaystack; *pos; pos++ ) 
	{
		if ( nextPos != NULL ) 
		{
			if ( toupper( *pos ) == toupper( *nextPos ) ) 
			{
				if ( !*++nextPos )
				{
					return startn;
				}
			} 
			else
			{
				nextPos = NULL;
			}
		}
		else if ( toupper( *pos ) == toupper( *inNeedle ) ) 
		{
			nextPos = inNeedle + 1;
			startn = pos;
		}
	}
	
	return NULL;
}

//-------------------------------------------------------------
// 関数： Jiro_GetStringLength
// 機能 : 全角半角を考慮して「文字数（日本語の”あ”を２バイトではなく１文字と判定）」を取得
// add_hisa 090513
//-------------------------------------------------------------
long Jiro_GetStringLength( const char* inStr )
{
	long	i, len;
	long	outMojisuu = 0;	// 文字数
	short	rc;
	
	// 文字列を１文字ずつ判定していく
	len = (long)strlen( inStr );
	for( i=0; i<len; i++ )
	{
		// 半角、もしくは全角２バイト目の場合、＋１する
		rc = Jiro_GetCharAttr( (unsigned char*)inStr, i );
		if( rc != 1 )
		{
			outMojisuu++;
		}
	}
	
	return outMojisuu;
}

//-------------------------------------------------------------
// 関数 : Jiro_GetTextEncoding
// 機能 : 文字エンコーディングを取得
// add_hisa 090730
//-------------------------------------------------------------
bool Jiro_GetTextEncoding( const char* inTextData, long inTextDataSize, char* outEncoding )
{
	long	encode;
	
	// エンコーディング判定
	encode = GetEncoding( inTextData, inTextDataSize );
	if( encode == 0 )
	{
		strcpy( outEncoding, kEncodingShiftJIS );	// アスキーの場合、シフトＪＩＳにする
		return true;
	}
	if( encode == 1 )
	{
		strcpy( outEncoding, kEncodingShiftJIS );	// シフトＪＩＳ
		return true;
	}
	if( encode == 2 )
	{
		strcpy( outEncoding, kEncodingUTF8 );	// ＵＴＦ８
		return true;
	}
	if( encode == 3 )
	{
		strcpy( outEncoding, kEncodingEUC );	// ＥＵＣ
		return true;
	}
	if( encode == 4 )
	{
		strcpy( outEncoding, kEncodingISO2022JP );	// ＩＳＯー２０２２ーＪＰ
		return true;
	}
	
	strcpy( outEncoding, kEncodingUnknown );	// それ以外は不明
	return false;
}

//-------------------------------------------------------------
// 関数 : GetEncoding
// 機能 : エンコーディングを取得
// ヒットしたコードの数が多いエンコーディングを返す
// ただしすべてアスキーコードなら、アスキーコードとして返す
// 　０＝アスキーコード
// 　１＝シフトＪＩＳ
// 　２＝ＵＴＦ８
// 　３＝ＥＵＣ
//   ４＝ＩＳＯー２０２２ーＪＰ
// −１＝不明
// add_hisa 090730
//-------------------------------------------------------------
static long GetEncoding( const char* inTextData, long inTextDataSize )
{
    long	numOfSJISCodes, numOfEUCCodes, numOfUTF8Codes;
	
	// ＩＳＯー２０２２ーＪＰ？
	if( IsISO2022JP( inTextData, inTextDataSize ) == true )
	{
		return 4;	// ＩＳＯー２０２２ーＪＰ
	}
	
	// すべてアスキーコードで構成されている？
	if( IsASCII( inTextData, inTextDataSize ) == true )
	{
		return 0;	// アスキーコード（シフトＪＩＳ）
	}
	
	// シフトＪＩＳ判定（−１ならシフトＪＩＳはありえない）
    numOfSJISCodes = CheckSJIS( inTextData, inTextDataSize );
	// ＵＴＦ８判定（−１ならＵＴＦ８はありえない）
	numOfUTF8Codes = CheckUTF8( inTextData, inTextDataSize );
	// ＥＵＣ判定（−１ならＥＵＣはありえない）
    numOfEUCCodes = CheckEUC( inTextData, inTextDataSize );
	
	// 一番シフトＪＩＳと判定されたコードが多い？
    if ( 0 < numOfSJISCodes && numOfEUCCodes < numOfSJISCodes && numOfUTF8Codes < numOfSJISCodes )
	{
        return 1;	// シフトＪＩＳ
	}
	// 一番シフトＵＴＦ８と判定されたコードが多い？
    else if ( 0 < numOfUTF8Codes && numOfEUCCodes < numOfUTF8Codes && numOfSJISCodes <= numOfUTF8Codes )
	{
        return 2;	// ＵＴＦ８
	}
	// 一番ＥＵＣと判定されたコードが多い？
	else if ( 0 < numOfEUCCodes && numOfSJISCodes <= numOfEUCCodes && numOfUTF8Codes <= numOfEUCCodes )
	{
        return 3;	// ＥＵＣ
	}
	
    return (-1);	// 不明
}

//-------------------------------------------------------------
// 関数 : CheckUTF8
// 機能 : ＵＴＦ８判定
// add_hisa 090730
//-------------------------------------------------------------
static long CheckUTF8( const char* inTextData, long inTextDataSize )
{
    unsigned char	byte1, byte2, byte3, byte4;
	long			i;
	long			outNumOfUTF8Codes = 0;
	bool			isBOM = false;
	
	// 文字コードの判定をしていく
    for( i=0; i<inTextDataSize; i++ ) 
    {
		// １バイト目がアスキーコード以外？
        byte1 = inTextData[i];
		if ( 0x7F < byte1 && i < (inTextDataSize-1) )
		{
			// ２バイト文字
			byte2 = inTextData[i + 1];
			if ((byte1 >= 0xC0 && byte1 <= 0xDF) && (byte2 >= 0x80 && byte2 <= 0xBF))
			{
				i = i + 1;
				outNumOfUTF8Codes = outNumOfUTF8Codes + 2;
			}
            else if ( i < (inTextDataSize-2) )
            {
				// ３バイト文字
                byte3 = inTextData[i + 2];
                if (byte1 == 0xEF && byte2 == 0xBB && byte3 == 0xBF)
                {
					isBOM = true;	// ＢＯＭ：0xEF 0xBB 0xBF
					i = i + 2;
					outNumOfUTF8Codes = outNumOfUTF8Codes + 3;
				}
				else if ( ( byte1 >= 0xE0 && byte1 <= 0xEF ) && 
						 ( byte2 >= 0x80 && byte2 <= 0xBF ) && 
						 ( byte3 >= 0x80 && byte2 <= 0xBF ) )
				{
					i = i + 2;
					outNumOfUTF8Codes = outNumOfUTF8Codes + 3;
				}
                else if ( i < (inTextDataSize-3) )
                {
					// ４バイト文字
                    byte4 = inTextData[i + 3];
                    if ((byte1 >= 0xF0 && byte1 <= 0xF7) &&
                        (byte2 >= 0x80 && byte2 <= 0xBF) &&
                        (byte3 >= 0x80 && byte2 <= 0xBF) &&
                        (byte4 >= 0x80 && byte2 <= 0xBF))
                    {
						i = i + 3;
						outNumOfUTF8Codes = outNumOfUTF8Codes + 4;
					}
                }
                else
                {
                    return (-1);
                }
            }
        }
    }
	
    return outNumOfUTF8Codes;
}

//-------------------------------------------------------------
// 関数 : CheckSJIS
// 機能 : シフトＪＩＳコード判定処理
// add_hisa 090730
//-------------------------------------------------------------
static long CheckSJIS( const char* inTextData, long inTextDataSize )
{
	unsigned char	byte1, byte2;
	long			outNumOfSJISCodes = 0;
	long			i;
	
	// 文字コードの判定をしていく
	for( i=0; i<inTextDataSize; i++ )
	{
		// １バイト目がアスキーコード？
		byte1 = inTextData[ i ];
		if ( 0x00 <= byte1 && byte1 <= 0x7F )
		{
			// アスキーコード（0x00〜0x7F）
		}
		else if( 0xA1 <= byte1 && byte1 <= 0xDF )
		{
			// 半角カタカナ（0xA1〜0xDF）
			outNumOfSJISCodes = outNumOfSJISCodes + 1;
		}
		// アスキーコード、半角カタカナ以外？
		else if( i < (inTextDataSize-1) )
		{
			// 第１バイト: 0x81〜0x9F、0xE0〜0xFC
			// 第２バイト: 0x40〜0x7E、0x80〜0xFC
			byte2 = inTextData[ i+1 ];
			if ( ( ( 0x81 <= byte1 && byte1 <= 0x9F ) || ( 0xE0 <= byte1 && byte1 <= 0xFC ) ) &&
				( ( 0x40 <= byte2 && byte2 <= 0x7E ) || ( 0x80 <= byte2 && byte2 <= 0xFC ) ) )
			{
				i++;
				outNumOfSJISCodes = outNumOfSJISCodes + 2;
			}
			else
			{
				return (-1);	// シフトＪＩＳではない
			}
		}
	}
	
	return outNumOfSJISCodes;
}

//-------------------------------------------------------------
// 関数 : CheckEUC
// 機能 : ＥＵＣコード判定処理
// add_hisa 090730
//-------------------------------------------------------------
static long CheckEUC( const char* inTextData, long inTextDataSize )
{
	unsigned char	byte1, byte2, byte3;
	long			outNumOfEUCCodes = 0;
	long			i;
	
	// 文字コードの判定をしていく
	for( i=0; i<inTextDataSize; i++ )
	{
		// １バイト目がアスキーコード以外？
		byte1 = inTextData[i];
		if ( 0x7F < byte1 && i < (inTextDataSize-1) )
		{
			byte2 = inTextData[i + 1];
			
			// 漢字：第１バイト（0xA1〜0xFE）、第２バイト（0xA1〜0xFE）？
			if( (byte1 >= 0xA1 && byte1 <= 0xFE) && (byte2 >= 0xA1 && byte1 <= 0xFE) )
			{
				i++;
				outNumOfEUCCodes = outNumOfEUCCodes + 2;
			}
			// 半角カタカナ：第１バイト（0x8E）, 第２バイト（0xA1〜0xDF）
			else if( (byte1 == 0x8E) && (byte2 >= 0xA1 && byte1 <= 0xDF) )
			{ 
				i++;
				outNumOfEUCCodes = outNumOfEUCCodes + 2;
			}
			// ３バイト目がある？
			else if ( i < (inTextDataSize-2) )
			{
				// 補助漢字：第１バイト（0x8F）、第２バイト（0xA1〜0xFE）、第3バイト（0xA1〜0xFE）
				byte3 = inTextData[i + 2];
				if ( (byte1 == 0x8F) && (byte2 >= 0xA1 && byte2 <= 0xFE) && (byte3 >= 0xA1 && byte3 <= 0xFE) )
				{ 
					i = i + 2;
					outNumOfEUCCodes = outNumOfEUCCodes + 3;
				}
				else
				{
					return (-1);	// ＥＵＣではない
				}
			}
			// それ以外
			else
			{
				return (-1);	// ＥＵＣではない
			}
		}
	}
	
	return outNumOfEUCCodes;
}

//-------------------------------------------------------------
// 関数 : IsASCII
// 機能 : アスキー判定
// add_hisa 090730
//-------------------------------------------------------------
static bool IsASCII( const char* inTextData, long inTextDataSize )
{
	unsigned char	byte1;
	long			i;
	
	// 文字コードの判定をしていく
    for( i=0; i<inTextDataSize; i++ ) 
    {
		// アスキーコード？
        byte1 = inTextData[i];
		if( 0x7E < byte1 )
        {
			return false;	// アスキーコード以外のコードがあった
        }
    }
	
    return true;	// すべてアスキーコード
}

//-------------------------------------------------------------
// 関数 : IsISO2022JP
// 機能 : ＩＳＯー２０２２ーＪＰ判定
// add_hisa 090818
//-------------------------------------------------------------
static bool IsISO2022JP( const char* inTextData, long inTextDataSize )
{
	unsigned char	byte1, byte2, byte3, byte4, byte5, byte6;
	long			i;
	
	// 文字コードの判定をしていく
    for( i=0; i<inTextDataSize; i++ ) 
	{
		// １バイト目がＩＳＯー２０２２ーＪＰ (0x00〜0x7F) の 範囲外
		byte1 = inTextData[ i ];
		if( byte1 > 0x7F )
		{
			return false;
		}
		else if( i < inTextDataSize-2 )
		{
			// エスケープシーケンス？
			byte2 = inTextData[ i+1 ];
			byte3 = inTextData[ i+2 ];
			if( byte1 == 0x1B && byte2 == 0x28 && byte3 == 0x42 )
			{
				return true;    // ＥＳＣ ( B ：JIS ASCII
			}
			else if( byte1 == 0x1B && byte2 == 0x28 && byte3 == 0x4A )
			{
				return true;    // ＥＳＣ ( J：JIS X 0201-1976 Roman Set
			}
			else if( byte1 == 0x1B && byte2 == 0x28 && byte3 == 0x49 )
			{
				return true;    // ＥＳＣ ( I：JIS X 0201-1976 片仮名
			}
			else if( byte1 == 0x1B && byte2 == 0x24 && byte3 == 0x40 )
			{
				return true;    // ＥＳＣ $ @：JIS X 0208-1978(通称：旧JIS)
			}
			else if( byte1 == 0x1B && byte2 == 0x24 && byte3 == 0x42 )
			{
				return true;    // ＥＳＣ $ B：JIS X 0208-1983(通称：新JIS)
			}
		}
		else if( i < inTextDataSize-3 )
		{
			// エスケープシーケンス？
			byte2 = inTextData[ i+1 ];
			byte3 = inTextData[ i+2 ]; 
			byte4 = inTextData[ i+3 ];
			if( byte1 == 0x1B && byte2 == 0x24 && byte3 == 0x28 && byte4 == 0x44 )
			{
				return true;    // ESC $ ( D：JIS X 0212-1990（JIS補助漢字）
			}
		}
		else if( i < inTextDataSize-5 )
		{
			// エスケープシーケンス？
			byte2 = inTextData[ i+1 ];
			byte3 = inTextData[ i+2 ];
			byte4 = inTextData[ i+3 ];
			byte5 = inTextData[ i+4 ];
			byte6 = inTextData[ i+5 ];
			if ( byte1 == 0x1B && byte2 == 0x26 && byte3 == 0x40 && byte4 == 0x1B && byte5 == 0x24 && byte6 == 0x42 )
			{
				return true;    // ESC & @ ESC $ B：JIS X 0208-1990
			}
		}
	}
	
	return false;	// ＩＳＯー２０２２ーＪＰではない
}
