// TYPE: C
// PRJ: JCL
// PGM: Jiro_TAB
// VER: 1.20
// USES: CodeWarrior
// CREATE: 1999/02/21( Shiratori )
// UPDATE: 2000/06/02( Shiratori )
// UPDATE: 2007/04/02( tamura )
// UPDATE: 2009/03/16( igari, tamura )
/////////////////////////////////////////////////////
// 000602 - ＭＳ-ＤＯＳ形式の改行にも対応
// 070402 - Linux版(CR/LF/CRLFに対応)

// MFCプロジェクトを使用する場合は、下記のコメントを外す。
//#include "stdafx.h"

// runtime
#include <string.h>
#include <stdlib.h>

//JCL
#include "Jiro_Const.h"
#include "Jiro_TAB.h"

//*************************************************************************
// 内部関数のプロトタイプ宣言
//*************************************************************************
static bool GetColumnData(char * rowStartPos,	// 行の先頭位置
							char * rowEndPos,		// 行の終了位置
							long	columnNo,		// 列番号（0から）
							char *	outBufPtr,		// データ出力先（C文字列）
							long	bufLength,		// 出力の最大長
							long * outRealLength)	// 実際の長さ
     ;
static void SetOutPutBuffer(char * textPos,
							long textLen, 
							char *	outBuf,
							long	bufLength,
							long *	outRealLength)
     ;
static char * SearchCRorLF( const char * inStr );
static char * SearchNextLine( const char * inStr );
static long GetNumOfColumns( char* inRowStartPos, char* inRowEndPos );

//*************************************************************************
// 関数
//*************************************************************************
//--------------------------------------------------
// 関数 : Jiro_TAB_Init
// 機能 : TABテーブルを初期化 (CSVデータをセット)
//--------------------------------------------------
void Jiro_TAB_Init( JIRO_TAB * outTabTblPtr, char * inTextPtr )
{
	// 文字列へのポインタをセット
	outTabTblPtr->textPtr = inTextPtr;

	// 文字列の最後(NULL文字)へのポインタをセット
	if( inTextPtr != NULL )
	{
		outTabTblPtr->textEndPos = inTextPtr + strlen( inTextPtr );
	}
	else
	{
		outTabTblPtr->textEndPos = NULL;
	}

	// 現在の行の位置
	outTabTblPtr->currentRowPos = NULL;
}

//--------------------------------------------------
// 関数 : SearchCRorLF
// 機能 : CRまたはLFを探す
// 返値 : 最初に見つかったCRまたはLFへのポインタ
//        見つからなかったらNULL
//--------------------------------------------------
static char * SearchCRorLF( const char * inStr )
{
	char * crPtr;
	char * lfPtr;

	// 最初のCRの位置を探す
	crPtr = strchr( (char*)inStr, kJIRO_CR );

	// 最初のLFの位置を探す
	lfPtr = strchr( (char*)inStr, kJIRO_LF );

	// CRがないときは、LFの位置を返す
	if( crPtr == NULL )
	{
		return lfPtr;
	}

	// LFがないときは、CRの位置を返す
	if( lfPtr == NULL )
	{
		return crPtr;
	}

	// CRとLFが見つかったときは、最初に見つかったほう
	if( crPtr < lfPtr )
	{ // CRが先
		return crPtr;
	}
	else
	{ // LFが先
		return lfPtr;
	}
}

//--------------------------------------------------
// 関数 : SearchNextLine
// 機能 : 次の行を探す
// 返値 : 次の行の先頭へのポインタ
//        見つからなかったらNULL
//--------------------------------------------------
static char * SearchNextLine( const char * inStr )
{
	char * charCRorLFPtr;
	char nextChar;
	char * nextLinePtr;

	// 最初のCRかLFを探す
	charCRorLFPtr = SearchCRorLF( inStr );

	// 見つからなかったらNULLを返す
	if( charCRorLFPtr == NULL )
	{
		return NULL;
	}

	// CR/LF/CRLFの次の文字を探す
	if( charCRorLFPtr[0] == kJIRO_CR )
	{ // CRだった
		// 次の文字を取得
		nextChar = charCRorLFPtr[1];

		// 次の文字がLFだったらCRLF
		if( nextChar == kJIRO_LF )
		{ // CRLFだった
			// CRLF分の2文字を飛ばす
			nextLinePtr = &( charCRorLFPtr[2] );
		}
		else
		{ // CRだった
			// CR分の1文字を飛ばす
			nextLinePtr = &( charCRorLFPtr[1] );
		}
	}
	else
	{ // LFだった
		// CR分の1文字を飛ばす
		nextLinePtr = &( charCRorLFPtr[1] );
	}

	return nextLinePtr;
}

//--------------------------------------------------
// 関数 : Jiro_TAB_GetRowCount
// 機能 : 行数を取得
// 説明 : CR/LF/CRLFに対応
//--------------------------------------------------
long Jiro_TAB_GetRowCount( JIRO_TAB * inTabTblPtr )
{
	char *	rowStartPos;  // 行の開始位置
	long	rowCnt;
	
	rowCnt = 0;
	rowStartPos = inTabTblPtr->textPtr;
	while ( rowStartPos < inTabTblPtr->textEndPos )
	{
		rowCnt += 1;

		// 次の行を探す
		rowStartPos = SearchNextLine( rowStartPos );
		if( rowStartPos == NULL )
		{
			break;
		}
	}
	return rowCnt;
}

//--------------------------------------------------
// 関数 : Jiro_TAB_FirstRow
// 機能 : 最初の行に移動
//--------------------------------------------------
bool Jiro_TAB_FirstRow( JIRO_TAB * inTabTblPtr )
{
	// 現在の位置をリセット
	inTabTblPtr->currentRowPos = NULL;
	if( inTabTblPtr->textPtr == NULL )
	{
		return false;
	}

	// memo tamura 070403 // なんで比較してるのかわからない
	if( inTabTblPtr->textPtr < inTabTblPtr->textEndPos )
	{
		inTabTblPtr->currentRowPos = inTabTblPtr->textPtr;
		return true;
	}
	return false;
}

//--------------------------------------------------
// 関数 : Jiro_TAB_NextRow
// 機能 : 次の行に移動
//--------------------------------------------------
bool Jiro_TAB_NextRow(JIRO_TAB * inTabTblPtr )
{
	char * nextLinePos;
	
	// NULLだったら移動しない
	if( inTabTblPtr->textPtr == NULL )
	{
		return false;
	}
	if( inTabTblPtr->currentRowPos == NULL )
	{
		return false;
	}

	// 次の行を探す
	nextLinePos = SearchNextLine( inTabTblPtr->currentRowPos );
	if( nextLinePos == NULL )
	{ // 見つからなかった
		inTabTblPtr->currentRowPos = NULL;
		return false;
	}
	else
	{ // 見つかった
		inTabTblPtr->currentRowPos = nextLinePos;
		return true;
	}
}

// -------------------------------------------------
// 関数 : Jiro_TAB_GetFieldData
// 機能 : 現在の行の指定された列のデータを取得
// inTabTblPtr (in) : TAB テーブル
// column (in) : 取得したいフィールドの番号(0 はじまり)
// bufLength (out) : フィールドの値のサイズ
// outBufPtr (out) : フィールドの値
// -------------------------------------------------
bool Jiro_TAB_GetFieldData( JIRO_TAB* inTabTblPtr, long inColumn, long inBufLength, char* outBufPtr )
{
	char *	rowEndPos;  // 行の終了位置 (改行文字かNULL文字)
	long	realLength;
	bool	rc;
	
	outBufPtr[0] = '\0';

	// NULLチェック
	if( inTabTblPtr->currentRowPos == NULL )
	{
		return false;
	}

	// 行の終わりを探す
	rowEndPos = SearchCRorLF( inTabTblPtr->currentRowPos );
	if( rowEndPos == NULL )
	{ // 見つからなかった
		// 文字列の終わりを行の終わりとする
		rowEndPos = inTabTblPtr->textEndPos;
	}

	// フィールドの値を取得する
	rc = GetColumnData( inTabTblPtr->currentRowPos, rowEndPos, inColumn, outBufPtr, inBufLength, &realLength );

	return rc;
}

//--------------------------------------------------
// 関数 : Jiro_TAB_GetFieldSize
// 機能 : 現在の行の指定された列の文字数を取得
//--------------------------------------------------
long Jiro_TAB_GetFieldSize( JIRO_TAB * inTabTblPtr, long inColumn )
{
	char *	rowEndPos;		// 行の終了位置
	long	realLength = 0L;

	// NULLチェック
	if( inTabTblPtr->currentRowPos == NULL )
	{
		return realLength;
	}

	// 行の終わりを探す
	rowEndPos = SearchCRorLF( inTabTblPtr->currentRowPos );
	if( rowEndPos == NULL )
	{ // 見つからなかった
		// 文字列の終わりを行の終わりとする
		rowEndPos = inTabTblPtr->textEndPos;
	}

	// フィールドの属性を取得する
	GetColumnData( inTabTblPtr->currentRowPos, rowEndPos, inColumn, NULL, 0, &realLength );

	// フィールドの文字数を返す。
	return realLength;
}

//--------------------------------------------------
// 関数 : GetColumnData
// 機能 : 現在の行の指定された列のデータを取得
//--------------------------------------------------
static bool GetColumnData(char * rowStartPos,		// 行の先頭位置
							char * rowEndPos,		// 行の終了位置
							long	columnNo,		// 列番号（0から）
							char *	outBufPtr,		// データ出力先（C文字列）
							long	bufLength,		// 出力の最大長
							long * outRealLength)	// 実際の長さ
{
	char *	targetPos;
	char *	columnStartPos;
	char *	columnEndPos;
	long	columnCnt;
	long	textLen;
	
	*outRealLength = 0;
	
	if (outBufPtr != NULL)
	{
		*outBufPtr = '\0';
	}

	columnCnt = 0;
	targetPos = rowStartPos;
	columnStartPos = rowStartPos;
	while (targetPos < rowEndPos)
	{
		if (*targetPos == kJIRO_TAB)	// TAB?
		{
			columnEndPos = targetPos;
			columnCnt++;
			if (columnCnt - 1 == columnNo)
			{
				// フィールドの値を設定
				textLen = (long)( columnEndPos - columnStartPos );
				SetOutPutBuffer( columnStartPos, textLen, outBufPtr, bufLength, outRealLength );
				return true;
			}
			
			targetPos++;
			columnStartPos = targetPos;
		}
		else
		{
			targetPos++;
			if (targetPos >= rowEndPos)
			{
				columnCnt++;
				columnEndPos = targetPos;
				if (columnCnt - 1 == columnNo)
				{
					// フィールドの値を設定
					textLen = (long)( columnEndPos - columnStartPos );
					SetOutPutBuffer(columnStartPos, textLen, outBufPtr, bufLength, outRealLength);
					return true;
				}
				else
				{
					return false;
				}
			}
		}
	}
	columnCnt++;
	if (columnCnt - 1 == columnNo)
	{
		return true;
	}

	return false;
}

// -------------------------------------------------
// 関数 : SetOutPutBuffer
// 機能 : フィールド値を設定
// -------------------------------------------------
static void SetOutPutBuffer(char * textPos,
							long textLen, 
							char *	outBuf,
							long	bufLength,
							long *	outRealLength)
{
	*outRealLength = textLen;
	if (outBuf != NULL)
	{
		if (textLen >= bufLength)
		{
			textLen = bufLength - 1;
		}
		
		memcpy(outBuf, textPos, textLen);
		outBuf[textLen] = '\0';
		*outRealLength = textLen;
	}
}

// -------------------------------------------------
// 関数 : Jiro_TAB_GetNumOfColumns
// 機能 : 現在の行の列数を取得
// add_igari 090316
// -------------------------------------------------
long Jiro_TAB_GetNumOfColumns( JIRO_TAB * inTabTblPtr )
{
	char*	rowEndPos;		// 行の終了位置
	long	numOfCols;
		
	if ( inTabTblPtr->currentRowPos == NULL )
	{
		return -1;
	}

	rowEndPos = strchr( inTabTblPtr->currentRowPos, kJIRO_CR );	// CRを検索
	if (rowEndPos == NULL)
	{
		rowEndPos = inTabTblPtr->textEndPos;
	}

	numOfCols = GetNumOfColumns( inTabTblPtr->currentRowPos, rowEndPos );

	return numOfCols;
}

//--------------------------------------------------
// 関数： GetNumOfColumns
// 機能： カレント行の列数を取得
// add_igari 090316
//--------------------------------------------------
static long GetNumOfColumns( char* inRowStartPos, char* inRowEndPos )
{
	char*	targetPos;
	long	columnCnt;
	
	columnCnt = 0;
	targetPos = inRowStartPos;
	while (targetPos < inRowEndPos)
	{
		if (*targetPos == kJIRO_TAB)	// TAB?
		{
			columnCnt++;
			targetPos++;
		}
		else
		{
			targetPos++;
			if (targetPos >= inRowEndPos)
			{
				columnCnt++;
				return columnCnt;
			}
		}
	}
	columnCnt++;
	
	return columnCnt;
}

//--------------------------------------------------
// 関数 : Jiro_TAB_GetFieldLongValue
// 機能 : 現在の行の指定された列のフィールドロング値を取得
// add_igari 090316
//--------------------------------------------------
long Jiro_TAB_GetFieldLongValue( JIRO_TAB * inTabTblPtr, long inColumn )
{
	char	longStr[ 32 ];
	long	outLongValue;
	
	memset( longStr, 0, sizeof( longStr ) );
	Jiro_TAB_GetFieldData( inTabTblPtr, inColumn, 32, longStr );
	outLongValue = atol( longStr );
	
	return outLongValue;
}

//--------------------------------------------------
// 関数 : Jiro_TAB_GetFieldDoubleValue
// 機能 : 現在の行の指定された列のフィールドダブル値を取得
// add_igari 090316
//--------------------------------------------------
double Jiro_TAB_GetFieldDoubleValue( JIRO_TAB * inTabTblPtr, long inColumn )
{
	char	doubleStr[ 32 ];
	double	outDoubleValue;
	
	memset( doubleStr, 0, sizeof( doubleStr ) );
	Jiro_TAB_GetFieldData( inTabTblPtr, inColumn, 32, doubleStr );
	outDoubleValue = atof( doubleStr );
	
	return outDoubleValue;
}


