// TYPE: C
// PRJ: JCL
// PGM: Jiro_File.cpp
// VER: 5.21
// USES: Xcode2.5.0, JCL
// CREATE: 2000/11/27( Siratori )
// UPDATE: 2003/09/15( hisa )
// UPDATE: 2005/07/01( Yamamoto, tsunoda )
// UPDATE: 2005/07/05( hisa )
// UPDATE: 2005/07/11( hisa )
// UPDATE: 2006/08/22( Asami, Sakurai )
// UPDATE: 2006/11/14( hisa )
// UPDATE: 2007/01/22( hisa )
// UPDATE: 2008/10/24( hisa )
// UPDATE: 2011/06/02( hisa )
/////////////////////////////////////////////////////
// ファイル操作関連の関数群
// 070122 hisa - Carbon Mach-Oに対応
// 081024 hisa - 次の関数をＪＣＬに昇格
// ・Jiro_GetFolderPath
// ・Jiro_GetFileName
// ・Jiro_RemoveExtension
// 110602 hisa - fprintf の引数をフォーマット形式に変更

// ansi
#include <stdio.h>
#include <string.h>
#include <stdarg.h>
#include <stdlib.h>
#include <time.h>
#include <dirent.h>

// JCL
#include "Jiro_File.h"
#include "Jiro_Const.h"

//*************************************************************************
// 内部関数のプロトタイプ宣言
//*************************************************************************

//*************************************************************************
// 関数
//*************************************************************************
//-----------------------------------------------------
// 関数 : Jiro_LogOut
// 機能 : ログファイルをデスクトップに出力する
// 070122 Hisa - タイム関数を変更。
//-----------------------------------------------------
void Jiro_LogOut( char* inFormat, ... )
{
	FILE*		fp;
	va_list 	vaList;
 	char		desktopPath[1024] = {0};
 	char		fullPath[1024] = {0};
	time_t		timer;
	struct tm*	timePtr;
	char		timeStr[128] = {0};
	OSErr		osErr;
	char		msg[65535] = {0};
	
	// デスクトップのパスを取得
	osErr = Jiro_GetDesktopPath( desktopPath );
	if( osErr != noErr )
	{
		return;
	}
	
	// ＵＮＩＸパスに変換してから、ファイルを開く
	sprintf( fullPath, "%slog.txt", desktopPath );
	fp = fopen( fullPath, "a" );
	if( fp == NULL )
	{
		return;
	}
	
	// 日付と時間を取得し、出力
	time( &timer );
	timePtr = localtime( &timer );
	if( timePtr != NULL )
	{
		sprintf( timeStr, " %04d/%02d/%02d %02d:%02d:%02d	"
					, timePtr->tm_year+1900, timePtr->tm_mon+1, timePtr->tm_mday
					, timePtr->tm_hour, timePtr->tm_min, timePtr->tm_sec );	
		fprintf( fp, "%s", timeStr );	// mod_hisa 110602
	}
	
	// メッセージを出力
	va_start( vaList, inFormat );
	vsprintf( msg, inFormat, vaList );
	fprintf( fp, "%s", msg );	// mod_hisa 110602
	va_end( vaList );
	
	// ファイルを閉じる	
	fclose( fp );
}

//-----------------------------------------------------
// 関数 : Jiro_LogDelete
// 機能 : デスクトップのログファイルを削除する
//-----------------------------------------------------
void Jiro_LogDelete( void ) //  mod takaesu 070730 引数無しのときはvoidで宣言
{
 	char		desktopPath[1024] = {0};
 	char		fullPath[1024] = {0};

	// デスクトップのパスを取得	
	if( Jiro_GetDesktopPath( fullPath ) == false )
	{
		return;
	}
	
	// ファイルを削除
	sprintf( fullPath, "%slog.txt", desktopPath );
	remove( fullPath );	
}

//-----------------------------------------------------
// 関数 : Jiro_GetDesktopPath
// 機能 : デスクトップのパスを取得
// 070122 Hisa - ＦＳＳｐｅｃをＦＳＲｅｆに変更
//-----------------------------------------------------
OSErr Jiro_GetDesktopPath( char* outDesktopFolderPath )
{
	OSErr		osErr;
	FSRef		fsRef;
	char		path[1024] = {0};
	
	// デスクトップＦＳＲｅｆを作成
	osErr = FSFindFolder( kOnSystemDisk, kDesktopFolderType, kDontCreateFolder, &fsRef );
	if( osErr != noErr )
	{
		return osErr;
	}
	
	// パスに変換
	osErr = FSRefMakePath( &fsRef, (UInt8*)path, 1024 );		// この関数で既にＵＮＩＸパスに変換済み
	if( osErr != noErr )
	{
		return osErr;
	}
	sprintf( outDesktopFolderPath , "%s/" , path ); // パスの最後に「/」を追加	

	return osErr;
}

//-----------------------------------------------------
// 関数 : Jiro_ReadBuffer_fromFile
// 機能 : ファイルをバッファに読み込む
//-----------------------------------------------------
long Jiro_ReadBuffer_fromFile( const char* inFilePath, char** outBuff )
{
	FILE *	fp = NULL;
	long	size;

	*outBuff = NULL;
	
	// ファイルを開く
	fp = fopen( inFilePath, "rb" );
	if ( fp == NULL )
	{
		return kFopen_Error;
	}
	
	// ファイルサイズを取得
	fseek( fp, 0, SEEK_END );
	size = ftell( fp );
	fseek( fp, 0L, SEEK_SET );
	if( size == (-1L) )
	{
		fclose( fp );
		return kFileSize_Error;
	}
	
	// バッファアロケート
	*outBuff = ( char* )malloc( size + 1 );
	if ( *outBuff == NULL )
	{
		fclose( fp );
		return kMalloc_Error;
	}
	
	// ファイルを読み込む
	memset( *outBuff, '\0', size+1 );
	fread( *outBuff, 1, size, fp );
	
	// ファイルを閉じる
	fclose( fp );
	
	return size;
}

//--------------------------------------------------
// 関数 : Jiro_GetAppFSSpec
// 機能 : app(実行可能ファイル)のFSSpecを取得
// add_hisa 060816
//--------------------------------------------------
void Jiro_GetAppFSSpec( FSSpec* outFileSpec )
{
	FSRef				processRef;
	FSCatalogInfo		processInfo;
	ProcessSerialNumber	processSerialNumber = { 0, kCurrentProcess };
	
	GetProcessBundleLocation( &processSerialNumber, &processRef );
	FSGetCatalogInfo( &processRef, kFSCatInfoNodeFlags, &processInfo, NULL, outFileSpec, NULL );
}

//-----------------------------------------------------
// 関数 : Jiro_GetAppDirectoryFSRef
// 機能 : .app(実行可能ファイル)があるディレクトリのＦＳＲｅｆを取得
//-----------------------------------------------------
void Jiro_GetAppDirectoryFSRef( FSRef* outAppDirFSRef )
{
	FSRef				currentFSRef;
	ProcessSerialNumber	processSerialNumber;

	// .app(実行可能ファイル)のパスを取得
	MacGetCurrentProcess( &processSerialNumber );
	GetProcessBundleLocation( &processSerialNumber, &currentFSRef );
	
	// .app(実行可能ファイル)があるディレクトリパスを取得
	FSGetCatalogInfo( &currentFSRef, kFSCatInfoNone, NULL, NULL, NULL, outAppDirFSRef );	
}

//-----------------------------------------------------
// 関数 : Jiro_GetAppDirectoryPath
// 機能 : app(実行可能ファイル)があるディレクトリパスを取得
// 060822 add_asami,sakurai ＦＳＲｅｆのみで動作するように変更
//-----------------------------------------------------
void Jiro_GetAppDirectoryPath( char* outUnixAppDirPath )
{
	FSRef		appDirFSRef;
	char		path[1024] = {0};
	
	// .app(実行可能ファイル)があるディレクトリパスを取得
	Jiro_GetAppDirectoryFSRef( &appDirFSRef );
	
	FSRefMakePath( &appDirFSRef, (UInt8*)path, 1024 );	// この関数で既にＵＮＩＸパスに変換済み
	sprintf( outUnixAppDirPath , "%s/" , path );		// パスの最後に「/」を追加
	// 「/Volumes」は必要な場合に自動的に追加される
	//　例 /Volumes/E/Tool/ 
}

//-----------------------------------------------------
// 関数 : Jiro_GetUserPreferencePath
// 機能 : ユーザの初期設定フォルダパスを取得
// add_igari 070208
//-----------------------------------------------------
Boolean Jiro_GetUserPreferencePath( char* outUserPreferencePath )
{
	OSErr		osErr;
	FSRef		fsRef;
	char		path[1024] = {0};
	
	// デスクトップＦＳＲｅｆを作成
	osErr = FSFindFolder( kOnSystemDisk, kPreferencesFolderType, kDontCreateFolder, &fsRef );
	if( osErr != noErr )
	{
		return false;
	}
	
	// パスに変換
	osErr = FSRefMakePath( &fsRef, (UInt8*)path, 1024 );	// この関数で既にＵＮＩＸパスに変換済み 
	if( osErr != noErr )
	{
		return false;
	}
	sprintf( outUserPreferencePath , "%s/" , path ); // パスの最後に「/」を追加	

	return true;
}

//------------------------------------------------
// 関数 : Jiro_IsFileExists
// 機能 : 指定ファイルが存在するか調べる
// 返値 : true ... 存在する、false ... 存在しない
// add_hisa, takaesu 071105
//------------------------------------------------
bool Jiro_IsFileExists( const char * inFilePath )
{
	FILE * fp;

	// ファイルを読み込みモードで試しに開く
	fp = fopen( inFilePath, "r" );
	if( fp == NULL )
	{
		return false;
	}
	
	fclose( fp );
	return true;
}

//------------------------------------------------
// 関数 : Jiro_GetFolderPath
// 機能 : ファイルパスからフォルダパスを切り出す
// メモ : inIsEndSeparator=trueの場合、最後にパス区切り文字が入る
// add_hisa 081024
//------------------------------------------------
void Jiro_GetFolderPath( const char* inFilePath, bool inIsEndSeparator, char* outFolderPath )
{
	char*	pos;
	long	findStrLen;
	long	foundLen;

	// 検索対象文字列の長さを取得
	findStrLen = (long) strlen( inFilePath );
	
	// 最後の「パス区切り文字」を探す
	pos = rindex( inFilePath, '/' );	// for Mac
	if( pos == NULL )
	{
		// そのまま返す
		strcpy( outFolderPath, inFilePath );
		return;
	}

	// ファイル名を外したパスから、最後の「パス区切り文字」を探す
	foundLen = strlen( pos );
	if( inIsEndSeparator == true )
	{
		foundLen = foundLen-1;
	}
	memcpy( outFolderPath, inFilePath, findStrLen - foundLen );
}

//------------------------------------------------
// 関数 : Jiro_GetFileName
// 機能 : ファイルパスからファイル名を取り出す
// add_hisa 081024
//------------------------------------------------
void Jiro_GetFileName( const char* inFilePath, char* outFileName )
{
	char* pos;
	
	// 最後の「パス区切り文字」を探す
	pos = rindex( inFilePath, '/' );	// for Mac
	if( pos == NULL )
	{ // '/'が入ってなかった
		// ファイルパスをファイル名とする
		strcpy( outFileName, inFilePath );
		return;
	}

	// '/'よりあとをファイル名とする
	strcpy( outFileName, pos + 1 );
}

//------------------------------------------------
// 関数 : Jiro_RemoveExtension
// 機能 : 拡張子を取り除く
// add_hisa 081024
//------------------------------------------------
void Jiro_RemoveExtension( const char* inFilePath, char* outOutExtFilePath )
{
	char*	pos;
	long	findStrLen;
	long	foundLen;

	// 検索対象文字列の長さを取得
	findStrLen = (long) strlen( inFilePath );
	
	// 最後の「拡張子」を探す
	pos = rindex( inFilePath, '.' );	// for Mac
	if( pos == NULL )
	{
		// そのまま返す
		strcpy( outOutExtFilePath, inFilePath );
		return;
	}

	// ファイル名を外したパスから、最後の「パス区切り文字」を探す
	foundLen = strlen( pos );
	memcpy( outOutExtFilePath, inFilePath, findStrLen - foundLen );
}

//------------------------------------------------
// 関数 : Jiro_IsFolder
// 機能 : フォルダかどうか
// add tamura 080205
//------------------------------------------------
bool Jiro_IsFolder( const char* inPath )
{
	DIR *			dir;
	
	// ディレクトリとして開こうとしてみる
	dir = opendir( inPath );
	if( dir == NULL )  // 開けなかった
	{
		// ディレクトリではない
		return false;
	}
	// 閉じる
	closedir( dir );
	
	// ディレクトリだった
	return true;
}

//------------------------------------------------
// 関数 : Jiro_GetExtension
// 機能 : 拡張子を取得
// add_hisa 081024
//------------------------------------------------
void Jiro_GetExtension( const char* inFilePath, char* outExt )
{
	char*	pos;
	long	findStrLen;

	// 検索対象文字列の長さを取得
	findStrLen = (long)strlen( inFilePath );
	
	// 最後の「拡張子」を探す
	pos = rindex( inFilePath, '.' );	// for Mac
	if( pos == NULL )
	{
		strcpy( outExt, "" );
		return;
	}
	
	// ファイル名を外したパスから、最後の「パス区切り文字」を探す
	pos++;
	strncpy( outExt, pos, 15 );
}

//-----------------------------------------------------
// 関数 : Jiro_GetApplicationSupportPath
// 機能 : ユーザのアプリケーションサポートフォルダパスを取得
// add_igari 070208
//-----------------------------------------------------
Boolean Jiro_GetApplicationSupportPath( char* outApplicationSupporPath )
{
	OSErr		osErr;
	FSRef		fsRef;
	char		path[1024] = {0};
	
	// デスクトップＦＳＲｅｆを作成
	osErr = FSFindFolder( kUserDomain, kApplicationSupportFolderType, kDontCreateFolder, &fsRef );
	if( osErr != noErr )
	{
		return false;
	}
	
	// パスに変換
	osErr = FSRefMakePath( &fsRef, (UInt8*)path, 1024 );	// この関数で既にＵＮＩＸパスに変換済み 
	if( osErr != noErr )
	{
		return false;
	}
	sprintf( outApplicationSupporPath , "%s/" , path ); // パスの最後に「/」を追加	

	return true;
}

//-----------------------------------------------------
// 関数 : Jiro_GetFileSize
// 機能 : ファイルサイズを取得
//-----------------------------------------------------
long Jiro_GetFileSize( const char* inFilePath )
{
	FILE*	fp;
	long	fsize = 0;

	// ファイルを開く
	fp = fopen( inFilePath, "rb" );
	if ( fp == NULL )
	{
		return (-1);
	}
	 
	// ファイルサイズを取得
	fseek( fp, 0, SEEK_END ); 
	fsize = ftell( fp );
	
	// ファイルを閉じる
	fclose( fp );
 
	return fsize;
}

//-----------------------------------------------------
// 関数 : Jiro_GetAppName
// 機能 : .app(実行可能ファイル)名を取得
//-----------------------------------------------------
void Jiro_GetAppName( char* outAppName )
{
	FSRef				currentFSRef;
	FSRef				appDirFSRef;
	ProcessSerialNumber	processSerialNumber;
	HFSUniStr255		name;
	CFStringRef			cfName;
	bool				rc;

	// .app(実行可能ファイル)のパスを取得
	MacGetCurrentProcess( &processSerialNumber );
	GetProcessBundleLocation( &processSerialNumber, &currentFSRef );
	
	// .app(実行可能ファイル)があるディレクトリパスを取得
	FSGetCatalogInfo( &currentFSRef, kFSCatInfoNone, NULL, &name, NULL, &appDirFSRef );	
	cfName = CFStringCreateWithCharacters( kCFAllocatorDefault, name.unicode, name.length ); 
 	rc = CFStringGetCString( cfName, outAppName, 128, kCFStringEncodingUTF8 );
	if( rc == false )
	{
		return;
	}	
}

//-----------------------------------------------------
// 関数 : Jiro_IsBinaryFile
// 機能 : バイナリファイル？
//-----------------------------------------------------
long Jiro_IsBinaryFile( const char* inFilePath )
{
	FILE*	fp;
	long 	code;

	// バイナリモードでファイルを開く
	fp = fopen( inFilePath, "rb" );
	if( fp == NULL )
	{
		return -1;
	}

	// ＥＯＦがくるまで
	while( feof( fp ) == 0 )
	{
		code = fgetc( fp );
		if( EOF == code )
		{
			break;	// ＥＯＦ
		}
		// アスキーコード０８以下（００は例外）？
		else if( 0 != code && code <= 8 )
		{
			fclose( fp );
			return 1;	// バイナリファイル
		}
	}
	fclose( fp );

	return 0;	// テキストファイル
}

//-----------------------------------------------------
// 関数 : Jiro_SystemErrorLog
// 機能 : ログファイルをアプリケーション横に出力する
//-----------------------------------------------------
void Jiro_SystemErrorLog( char* inFormat, ... )
{
	FILE*		fp;
	va_list 	vaList;
 	char		appFolderPath[1024] = {0};
 	char		fullPath[1024] = {0};
	time_t		timer;
	struct tm*	timePtr;
	char		timeStr[128] = {0};
	char		msg[65535] = {0};
	
	// デスクトップのパスを取得
	Jiro_GetAppDirectoryPath( appFolderPath );
	
	// ＵＮＩＸパスに変換してから、ファイルを開く
	sprintf( fullPath, "%sbunkoban_log.txt", appFolderPath );
	fp = fopen( fullPath, "a" );
	if( fp == NULL )
	{
		return;
	}
	
	// 日付と時間を取得し、出力
	time( &timer );
	timePtr = localtime( &timer );
	if( timePtr != NULL )
	{
		sprintf( timeStr, " %04d/%02d/%02d %02d:%02d:%02d	"
					, timePtr->tm_year+1900, timePtr->tm_mon+1, timePtr->tm_mday
					, timePtr->tm_hour, timePtr->tm_min, timePtr->tm_sec );	
		fprintf( fp, "%s", timeStr );	// mod_hisa 110602
	}
	
	// メッセージを出力
	va_start( vaList, inFormat );
	vsprintf( msg, inFormat, vaList );
	fprintf( fp, "%s", msg );	// mod_hisa 110602
	va_end( vaList );
	
	// ファイルを閉じる	
	fclose( fp );
}


