// TYPE: C
// PRJ: JCL
// PGM: Jiro_File.cpp
// VER: 5.21
// USES: Xcode2.5.0, JCL
// CREATE: 2000/11/27( Siratori )
// UPDATE: 2003/09/15( hisa )
// UPDATE: 2005/07/01( Yamamoto, tsunoda )
// UPDATE: 2005/07/05( hisa )
// UPDATE: 2005/07/11( hisa )
// UPDATE: 2006/08/22( Asami, Sakurai )
// UPDATE: 2006/11/14( hisa )
// UPDATE: 2007/01/22( hisa )
// UPDATE: 2008/10/24( hisa )
// UPDATE: 2011/06/02( hisa )
/////////////////////////////////////////////////////
// ファイル操作関連の関数群
// 070122 hisa - Carbon Mach-Oに対応
// 081024 hisa - 次の関数をＪＣＬに昇格
// ・Jiro_GetFolderPath
// ・Jiro_GetFileName
// ・Jiro_RemoveExtension
// 110602 hisa - fprintf の引数をフォーマット形式に変更
#ifndef __JIRO_FILE__
#define __JIRO_FILE__

// add_hisa 061114
// FSFindFolder()や CFStringCreateWithCharacters() などのカーボンＡＰＩを使用する場合、
// 「Carbon.h」をインクルードする必要があります。
// Carbon.hは「Carbon.framework」に含まれています。
// #import <Carbon/Carbon.h>を記述して、フレームワークを読み込みます。
#import <Carbon/Carbon.h>	// add_hisa 061114

// add_hisa 061114
// MacGetCurrentProcess()や GetProcessBundleLocation() などのプロセス関連ＡＰＩを使用する場合、
// 「Processes.h」をインクルードする必要があります。
// Processes.hは「ApplicationServices.framework」に含まれています。
// #import <ApplicationServices/ApplicationServices.h>を記述して、フレームワークを読み込みます。
#import <ApplicationServices/ApplicationServices.h>	// add_hisa 061114

#ifdef __cplusplus
extern "C" {
#endif

//***********************************************************
// 定数
//***********************************************************
#define	kLogFileName		"log.txt"	// ファイル名
#define kFopen_Error		(-1)		// ファイルオープンエラー
#define kFileSize_Error		(-2)		// ファイルサイズエラー
#define kMalloc_Error		(-3)		// メモリ確保エラー

//***********************************************************
// 構造体
//***********************************************************

//***********************************************************
// 外部関数のプロトタイプ宣言
//***********************************************************
//-------------------------------------------
// 関数 : Jiro_LogOut
// 機能 : 時間付きログファイルを出力
//-------------------------------------------
void Jiro_LogOut( char * inFormat, ... );

//-------------------------------------------
// 関数 : LogDelete
// 機能 : 出力したログファイルを削除
//-------------------------------------------
void Jiro_LogDelete( void ); //  mod takaesu 070730 引数無しのときはvoidで宣言

//-----------------------------------------------------
// 関数 : Jiro_GetDesktopPath
// 機能 : デスクトップのパスを取得
//-----------------------------------------------------
OSErr Jiro_GetDesktopPath( char* outFilePath );

//-----------------------------------------------------
// 関数 : Jiro_ReadBuffer_fromFile
// 機能 : ファイルをバッファに読み込む
//-----------------------------------------------------
long Jiro_ReadBuffer_fromFile( const char* inFilePath, char** outBuff );

//-----------------------------------------------------
// 関数 : Jiro_GetAppDirectoryFSRef
// 機能 : .app(実行可能ファイル)があるディレクトリパスを取得
//-----------------------------------------------------
void Jiro_GetAppDirectoryFSRef( FSRef* outAppDirFSRef );

//-----------------------------------------------------
// 関数 : Jiro_GetAppDirectoryPath
// 機能 : app(実行可能ファイル)があるディレクトリパスを取得
//-----------------------------------------------------
void Jiro_GetAppDirectoryPath( char* outUnixAppDirPath );

//-----------------------------------------------------
// 関数 : Jiro_GetUserPreferencePath
// 機能 : ユーザの初期設定フォルダパスを取得
// add_igari 070208
//-----------------------------------------------------
Boolean Jiro_GetUserPreferencePath( char* outUserPreferencePath );

//--------------------------------------------------
// 関数 : Jiro_GetAppFSSpec
// 機能 : app(実行可能ファイル)のFSSpecを取得
// add_hisa 060816
//--------------------------------------------------
void Jiro_GetAppFSSpec( FSSpec* outFileSpec );

//------------------------------------------------
// 関数 : Jiro_IsFileExists
// 機能 : 指定ファイルが存在するか調べる
// 返値 : true ... 存在する、false ... 存在しない
// add_hisa, takaesu 071105
//------------------------------------------------
bool Jiro_IsFileExists( const char * inFilePath );

//------------------------------------------------
// 関数 : Jiro_GetFolderPath
// 機能 : ファイルパスからフォルダパスを切り出す
// メモ : inIsEndSeparator=trueの場合、最後にパス区切り文字が入る
// add_hisa 081024
//------------------------------------------------
void Jiro_GetFolderPath( const char* inFilePath, bool inIsEndSeparator, char* outFolderPath );

//------------------------------------------------
// 関数 : Jiro_GetFileName
// 機能 : ファイルパスからファイル名を取り出す
// add_hisa 081024
//------------------------------------------------
void Jiro_GetFileName( const char* inFilePath, char* outFileName );

//------------------------------------------------
// 関数 : Jiro_RemoveExtension
// 機能 : 拡張子を取り除く
// add_hisa 081024
//------------------------------------------------
void Jiro_RemoveExtension( const char* inFilePath, char* outOutExtFilePath );

//------------------------------------------------
// 関数 : Jiro_IsFolder
// 機能 : フォルダかどうか
// add tamura 080205
//------------------------------------------------
bool Jiro_IsFolder( const char* inPath );

//------------------------------------------------
// 関数 : Jiro_GetExtension
// 機能 : 拡張子を取得
// add_hisa 081024
//------------------------------------------------
void Jiro_GetExtension( const char* inFilePath, char* outExt );

//-----------------------------------------------------
// 関数 : Jiro_GetApplicationSupportPath
// 機能 : ユーザのアプリケーションサポートフォルダパスを取得
// add_igari 070208
//-----------------------------------------------------
Boolean Jiro_GetApplicationSupportPath( char* outApplicationSupporPath );

//-----------------------------------------------------
// 関数 : Jiro_GetFileSize
// 機能 : ファイルサイズを取得
//-----------------------------------------------------
long Jiro_GetFileSize( const char* inFilePath );

//-----------------------------------------------------
// 関数 : Jiro_GetAppName
// 機能 : .app(実行可能ファイル)名を取得
//-----------------------------------------------------
void Jiro_GetAppName( char* outAppName );

//-----------------------------------------------------
// 関数 : Jiro_IsBinaryFile
// 機能 : バイナリファイル？
//-----------------------------------------------------
long Jiro_IsBinaryFile( const char* inFilePath );

//-----------------------------------------------------
// 関数 : Jiro_SystemErrorLog
// 機能 : ログファイルをアプリケーション横に出力する
//-----------------------------------------------------
void Jiro_SystemErrorLog( char* inFormat, ... );

#ifdef __cplusplus
}
#endif

#endif
